@@ -439,8 +439,8 @@ func (c *Conn) checkSPF(addr string) (spf.Result, error) {
 	}
 
 	if tcp, ok := c.conn.RemoteAddr().(*net.TCPAddr); ok {
-		res, err := spf.CheckHost(
-			tcp.IP, envelope.DomainOf(addr))
+		res, err := spf.CheckHostWithSender(
+			tcp.IP, envelope.DomainOf(addr), addr)
 
 		c.tr.Debugf("SPF %v (%v)", res, err)
 		spfResultCount.Add(string(res), 1)
@@ -604,9 +604,10 @@ func (c *Conn) DATA(params string) (code int, msg string) {
 
 	// There are no partial failures here: we put it in the queue, and then if
 	// individual deliveries fail, we report via email.
+	// If we fail to queue, return a transient error.
 	msgID, err := c.queue.Put(c.mailFrom, c.rcptTo, c.data)
 	if err != nil {
-		return 554, fmt.Sprintf("5.3.0 Failed to queue message: %v", err)
+		return 451, fmt.Sprintf("4.3.0 Failed to queue message: %v", err)
 	}
 
 	c.tr.Printf("Queued from %s to %s - %s", c.mailFrom, c.rcptTo, msgID)
@@ -741,7 +742,6 @@ func (c *Conn) runPostDataHook(data []byte) ([]byte, bool, error) {
 	}
 	tr := trace.New("Hook.Post-DATA", c.conn.RemoteAddr().String())
 	defer tr.Finish()
-	tr.Debugf("running")
 
 	ctx, cancel := context.WithTimeout(context.Background(), 1*time.Minute)
 	defer cancel()
@@ -769,14 +769,14 @@ func (c *Conn) runPostDataHook(data []byte) ([]byte, bool, error) {
 	cmd.Env = append(cmd.Env, "SPF_PASS="+boolToStr(c.spfResult == spf.Pass))
 
 	out, err := cmd.Output()
+	tr.Debugf("stdout: %q", out)
 	if err != nil {
 		hookResults.Add("post-data:fail", 1)
 		tr.Error(err)
-		tr.Debugf("stdout: %s", out)
 
 		permanent := false
 		if ee, ok := err.(*exec.ExitError); ok {
-			tr.Printf("stderr: %s", string(ee.Stderr))
+			tr.Printf("stderr: %q", string(ee.Stderr))
 			if status, ok := ee.Sys().(syscall.WaitStatus); ok {
 				permanent = status.ExitStatus() == 20
 			}
@@ -792,12 +792,11 @@ func (c *Conn) runPostDataHook(data []byte) ([]byte, bool, error) {
 	// contents. If it does not, just skip it.
 	if !isHeader(out) {
 		hookResults.Add("post-data:badoutput", 1)
-		tr.Errorf("error parsing post-data output: '%s'", out)
+		tr.Errorf("error parsing post-data output: %q", out)
 		return nil, false, nil
 	}
 
 	tr.Debugf("success")
-	tr.Debugf("stdout: %s", out)
 	hookResults.Add("post-data:success", 1)
 	return out, false, nil
 }
